@@ -1,6 +1,8 @@
 import kaa
 from kaa.command import Commands, command, is_enable, norec, norerun
 from kaa import document
+
+#todo: following imports should be defered
 from kaa.ui.mainmenu import mainmenumode
 from kaa.ui.itemlist import itemlistmode
 from kaa.ui.moveseparator import moveseparatormode
@@ -20,6 +22,13 @@ class ApplicationCommands(Commands):
         doc = mainmenumode.FileMenuMode.build(wnd)
         kaa.app.show_menu(wnd, doc, root=False)
 
+    @command('menu.recently-used-files')
+    @norec
+    @norerun
+    def show_recentfilesmenu(self, wnd):
+        doc = mainmenumode.RecentlyUsedFileMenuMode.build(wnd)
+        kaa.app.show_menu(wnd, doc, root=False)
+
     @command('menu.edit')
     @norec
     @norerun
@@ -115,7 +124,6 @@ class ApplicationCommands(Commands):
                 n = len(wnds)-1
             wnds[n].activate()
 
-
     @command('editor.joinwindow')
     @norec
     def editor_joinwindow(self, wnd):
@@ -137,47 +145,40 @@ class ApplicationCommands(Commands):
 
             wnd.document.mode.file_commands.save_documents(wnd, saves, saved)
 
-
-        #if wnd.splitter and wnd.splitter.parent:
-        #    if wnd.splitter.parent.wnd:
-        #        # not splitted
-        #        return
-        #
-        #    def saved():
-        #        wnd.splitter.parent.join(wnd)
-        #
-        #    if len(wnd.document.wnds) == 1:
-        #        wnd.document.mode.file_commands.ask_doc_close(
-        #            wnd, wnd.document, saved)
-        #    else:
-        #        saved()
-
     @command('editor.switchfile')
     @norec
     @norerun
     def editor_switchfile(self, wnd):
         docs = []
         for frame in kaa.app.get_frames():
-            for splitter, w in frame.splitter.walk():
-                if w:
-                    doc = w.document
-                    if doc not in docs:
-                        docs.append(doc)
+            docs.extend(frame.get_documents())
 
         titles = [doc.get_title() for doc in docs]
 
+        curdoc = wnd.document
+        curdoc_close = curdoc.mode.CLOSE_ON_DEL_WINDOW
+
         def callback(n):
+            curdoc.mode.CLOSE_ON_DEL_WINDOW = curdoc_close
+            if n is not None:
+                doc = docs[n]
+                wnd.show_doc(doc)
+            else:
+                wnd.show_doc(curdoc)
+
+        def selchanged(n):
             doc = docs[n]
             wnd.show_doc(doc)
 
         def saved():
+            curdoc.mode.CLOSE_ON_DEL_WINDOW = False
             n = docs.index(wnd.document)
-            doc = itemlistmode.ItemListMode.build('', titles, n, callback)
+            doc = itemlistmode.ItemListMode.build('', titles, n, callback, selchanged)
             kaa.app.show_dialog(doc)
 
         if len(wnd.document.wnds) == 1:
             wnd.document.mode.file_commands.ask_doc_close(
-                wnd, wnd.document, saved)
+                wnd, wnd.document, saved, 'Save file before close?')
         else:
             saved()
 
