@@ -1,4 +1,4 @@
-import shlex
+import re
 import subprocess
 import unicodedata
 import pyjf3
@@ -69,6 +69,39 @@ class CursorCommands(Commands):
     def last(self, wnd):
         wnd.cursor.eof()
 
+    @command('cursor.go-to-line')
+    @norerun
+    @norec
+    def go_to_line(self, wnd):
+        def callback(w, s):
+            s = s.strip()
+            try:
+                lineno = int(s)
+            except ValueError as e:
+                kaa.app.messagebar.set_message(str(e))
+                return
+
+            if lineno == 0 or lineno > wnd.document.buf.lineno.linecount():
+                kaa.app.messagebar.set_message('Enter valid line number.')
+                return
+
+            pos = wnd.document.get_lineno_pos(lineno)
+            tol = wnd.document.gettol(pos)
+            wnd.cursor.setpos(wnd.cursor.adjust_nextpos(wnd.cursor.pos, tol))
+
+            popup = w.get_label('popup')
+            popup.destroy()
+
+
+        def filter(wnd, s):
+            return re.match(r'\d*', s).group()
+
+        from kaa.ui.inputline import inputlinemode
+        doc = inputlinemode.InputlineMode.build('Line number:', callback, filter=filter)
+        kaa.app.messagebar.set_message("Enter line number")
+
+        kaa.app.show_dialog(doc)
+
 
 class ScreenCommands(Commands):
     @command('screen.selection.begin')
@@ -86,18 +119,62 @@ class ScreenCommands(Commands):
     def selection_clear(self, wnd):
         wnd.screen.selection.clear()
 
-    @command('screen.lineselection.curline')
+    @command('screen.selection.all')
+    @norerun
+    def select_all(self, wnd):
+        f = wnd.cursor.adjust_nextpos(wnd.cursor.pos, 0)
+        t = wnd.cursor.adjust_nextpos(wnd.cursor.pos, wnd.document.endpos())
+        wnd.screen.selection.set_range(f, t)
+
+    @command('screen.selection.curline')
     @norerun
     def select_cur_line(self, wnd):
-        tol = wnd.document.gettol(wnd.cursor.pos)
-        eol = wnd.document.geteol(tol)
+        tol = wnd.cursor.adjust_nextpos(
+                wnd.cursor.pos,
+                wnd.document.gettol(wnd.cursor.pos))
+        eol = wnd.cursor.adjust_nextpos(
+                wnd.cursor.pos,
+                wnd.document.geteol(tol))
 
         wnd.screen.selection.set_range(tol, eol)
 
+    @command('screen.selection.curword')
+    @norerun
+    def select_cur_word(self, wnd):
+        f, t = wnd.document.mode.get_word_at(wnd.cursor.pos)
+
+        f = wnd.cursor.adjust_nextpos(wnd.cursor.pos, f)
+        t = wnd.cursor.adjust_nextpos(wnd.cursor.pos, t)
+
+        wnd.screen.selection.set_range(f, t)
+
+    @command('screen.selection.expand_sel')
+    @norerun
+    @norec
+    def expand_sel(self, wnd):
+        mode = wnd.document.mode
+        keys = wnd.editmode.last_command_keys
+        L = len(keys)
+
+        if L >= 3 and keys[-1] == keys[-2] == keys[-3]:
+            self.select_all(wnd)
+            if kaa.app.macro.recording:
+                kaa.app.macro.record(self.select_all)
+
+        elif L >= 2 and (keys[-1] == keys[-2]):
+            self.select_cur_line(wnd)
+            if kaa.app.macro.recording:
+                kaa.app.macro.record(self.select_cur_line)
+
+        else:
+            self.select_cur_word(wnd)
+            if kaa.app.macro.recording:
+                kaa.app.macro.record(self.select_cur_word)
+
     @command('screen.lineselection.begin')
     @norerun
     def lineselection_begin(self, wnd):
-        tol = wnd.document.gettol(wnd.cursor.pos)
+        tol = wnd.cursor.adjust_nextpos(wnd.document.gettol(wnd.cursor.pos))
         wnd.screen.selection.start_selection(tol)
 
     @command('screen.lineselection.set-end')
@@ -109,10 +186,12 @@ class ScreenCommands(Commands):
         else:
             pos = wnd.cursor.pos
             if pos < f:
-                tol = wnd.document.gettol(pos)
+                tol = wnd.cursor.adjust_nextpos(
+                        wnd.document.gettol(pos))
                 wnd.screen.selection.set_end(tol)
             else:
-                eol = wnd.document.geteol(pos)
+                eol = wnd.cursor.adjust_nextpos(
+                        wnd.document.geteol(pos))
                 wnd.screen.selection.set_end(eol)
 
 
@@ -122,7 +201,7 @@ class EditCommands(Commands):
      UNDO_DELETE) = range(3)
 
     def on_edited(self, wnd):
-        pass
+        wnd.document.mode.on_edited(wnd)
 
     def insert_string(self, wnd, pos, s, update_cursor=True):
         """Insert string"""
@@ -172,12 +251,14 @@ class EditCommands(Commands):
                 wnd.cursor.setpos(pos)
                 wnd.cursor.savecol()
 
-        if wnd.document.undo:
-            wnd.document.undo.add(self.UNDO_DELETE, pos, posto, deled,
-                                  cur_pos, wnd.cursor.pos)
-        self.on_edited(wnd)
+            if wnd.document.undo:
+                wnd.document.undo.add(self.UNDO_DELETE, pos, posto, deled,
+                                      cur_pos, wnd.cursor.pos)
+            self.on_edited(wnd)
 
     def put_string(self, wnd, s):
+        s = wnd.document.mode.filter_string(wnd, s)
+
         sel = wnd.screen.selection.get_range()
         wnd.screen.selection.clear()
         if sel:
@@ -235,7 +316,7 @@ class EditCommands(Commands):
         pos = wnd.cursor.pos
         f = wnd.cursor.adjust_nextpos(pos, wnd.document.gettol(pos))
         t = wnd.cursor.adjust_nextpos(pos, wnd.document.geteol(f))
-        _trace(wnd.document.gettol(pos), wnd.document.geteol(pos))
+
         if f < t:
             self.delete_string(wnd, f, t)
 
@@ -287,10 +368,11 @@ class EditCommands(Commands):
 
     def _get_line_sel(self, wnd):
         doc = wnd.document
-        if not wnd.screen.selection.is_started():
+        sel = wnd.screen.selection.get_range()
+        if not sel:
             f = t = wnd.cursor.pos
         else:
-            f, t = wnd.screen.selection.get_range()
+            f, t = sel
 
         tol =  doc.gettol(f)
 
@@ -643,6 +725,7 @@ class RerunCommand(Commands):
             if not command:
                 msg = 'command {!r} is not registered.'.format(commandid)
                 kaa.app.messagebar.set_message(msg)
+                kaa.log.error(msg)
                 return
             command(wnd)
 
