@@ -15,13 +15,31 @@ class FileCommands(Commands):
     @command('file.open')
     @norec
     @norerun
-    def file_open(self, wnd):
+    def file_open(self, wnd, filename=''):
         def cb(filename, encoding, newline):
+            if not filename:
+                return
+
+            for frame in kaa.app.get_frames():
+                for doc in frame.get_documents():
+                    if doc.get_filename() == filename:
+                        if doc.wnds:
+                            wnd = doc.wnds[0]
+                            wnd.get_label('frame').bring_top()
+                            wnd.activate()
+
+                        MAX_FILENAMEMSG_LEN = 50
+                        if len(filename) > MAX_FILENAMEMSG_LEN:
+                            filename = '...{}'.format(filename[MAX_FILENAMEMSG_LEN*-1:])
+
+                        kaa.app.messagebar.set_message("`{}` is already opened.".format(filename))
+                        return
+
             doc = kaa.app.storage.openfile(filename, encoding, newline)
             kaa.app.show_doc(doc)
 
         from kaa.ui.selectfile import selectfile
-        selectfile.show_fileopen('.', cb)
+        selectfile.show_fileopen(filename, cb)
 
     @command('file.info')
     @norec
@@ -69,8 +87,9 @@ class FileCommands(Commands):
     def file_saveas(self, wnd, saved=None, document=None):
         "Show save_as dialog and save to the specified file."
         def cb(filename, enc, newline):
-            self.file_save(wnd, filename, saved=saved, document=document,
-                           encoding=enc, newline=newline)
+            if filename:
+                self.file_save(wnd, filename, saved=saved, document=document,
+                               encoding=enc, newline=newline)
 
         if not document:
             document = wnd.document
@@ -80,44 +99,43 @@ class FileCommands(Commands):
                                         document.fileinfo.newline,
                                         document.fileinfo.encoding, cb)
 
-    def ask_doc_close(self, wnd, document, callback):
+    def ask_doc_close(self, wnd, document, callback, msg):
         def saved():
            callback()
 
         def choice(c):
             if c == 'y':
-                self.file_save(wnd, saved=saved)
+                self.file_save(wnd, saved=saved, document=document)
             elif c == 'n':
                 callback()
 
         if document.undo and document.undo.is_dirty():
             msgboxmode.MsgBoxMode.show_msgbox(
-                'Save file before close? [{}]: '.format(
-                    document.get_title()),
+                '{} [{}]: '.format(
+                    msg, document.get_title()),
                 ['&Yes', '&No', '&Cancel'], choice)
         else:
             callback()
 
-    def save_documents(self, wnd, docs, callback):
+    def save_documents(self, wnd, docs, callback, msg='', force=False):
+
         docs = list(docs)
-        def save_documents():
+        def _save_documents():
             if not docs:
                 if callback:
                     callback()
             else:
                 doc = docs.pop()
-                self.ask_doc_close(wnd, doc, save_documents)
-        save_documents()
+                if force:
+                    self.file_save(wnd, saved=_save_documents, document=doc)
+                else:
+                    self.ask_doc_close(wnd, doc, _save_documents, msg)
 
-    @command('file.close')
-    @norec
-    @norerun
-    def file_close(self, wnd):
-        "Close active frame"
+        _save_documents()
 
-        import kaa.fileio
 
-        frame = wnd.get_label('frame')
+    def _close_frame(self, frame, callback):
+
         editors = {e for e in frame.get_editors()}
         alldocs = {e.document for e in editors}
 
@@ -127,28 +145,100 @@ class FileCommands(Commands):
             if not doc_editors:
                 docs.append(doc)
 
-        def saved():
+        def cb():
             frame.destroy()
+            callback()
+    
+        self.save_documents(frame, docs, cb, 'Save file before close?')
 
+    @command('file.close')
+    @norec
+    @norerun
+    def file_close(self, wnd, callback=None):
+        "Close active frame"
+
+        import kaa.fileio
+
+        frame = wnd.get_label('frame')
+        def saved():
             if frame.mainframe.childframes:
-                kaa.app.set_focus(frame.mainframe.childframes[-1])
+                f = frame.mainframe.childframes[0]
+                f.bring_top()
+                kaa.app.set_focus(f)
             else:
-                doc = kaa.fileio.newfile()
+                doc = kaa.fileio.newfile(provisional=True)
                 kaa.app.show_doc(doc)
 
-        self.save_documents(wnd, docs, saved)
+        self._close_frame(frame, saved)
+        
+    def get_current_documents(self, wnd):
+        docs = set()
+        for frame in wnd.mainframe.childframes:
+            editors = {e for e in frame.get_editors()}
+            docs |= {e.document for e in editors if e.document.mode.DOCUMENT}
+        return docs
+        
+    @command('file.save.all')
+    @norec
+    @norerun
+    def file_saveall(self, wnd):
+
+        def saved():
+            pass
+        
+        docs = self.get_current_documents(wnd)
+        docs = [doc for doc in docs if doc.undo.is_dirty()]
+        if docs:
+            self.save_documents(wnd, docs, saved, force=True)
+
+    @command('file.close.all')
+    @norec
+    @norerun
+    def file_closeall(self, wnd):
+
+        def callback():
+            import kaa.fileio
+            if frames:
+                f = frames.pop()
+                self._close_frame(f, callback)
+            else:
+                doc = kaa.fileio.newfile(provisional=True)
+                kaa.app.show_doc(doc)
+                
+        frames = wnd.mainframe.childframes[:]
+        callback()
+
+    @command('file.recently-used-files')
+    @norec
+    @norerun
+    def file_recently_used_files(self, wnd):
+
+        def cb(filename):
+            if filename:
+                self.file_open(wnd, filename)
+
+        from kaa.ui.recentlyusedfiles import recentlyusedfilesmode
+        recentlyusedfilesmode.show_recentlyusedfiles(cb)
+
+    @command('file.recently-used-directories')
+    @norec
+    @norerun
+    def file_recently_used_dirs(self, wnd):
+
+        def cb(filename):
+            if filename:
+                self.file_open(wnd, filename)
+
+        from kaa.ui.recentlyusedfiles import recentlyusedfilesmode
+        recentlyusedfilesmode.show_recentlyuseddirs(cb)
 
     @command('file.quit')
     @norec
     @norerun
     def file_quit(self, wnd):
 
-        docs = set()
-        for frame in wnd.mainframe.childframes:
-            editors = {e for e in frame.get_editors()}
-            docs |= {e.document for e in editors}
-
         def saved():
             kaa.app.quit()
-
-        self.save_documents(wnd, docs, saved)
+        
+        docs = self.get_current_documents(wnd)
+        self.save_documents(wnd, docs, saved, 'Save file before close?')
