@@ -5,6 +5,11 @@ KAA_SCRIPT = '__kaa__.py'
 LOGDIR = 'log'
 HISTDIR = 'history'
 
+HIST_FILENAME = 'files'
+HIST_DIRNAME = 'dirs'
+HIST_SEARCH = 'searchstr'
+HIST_REPLACE = 'replstr'
+
 NEWLINES = ['auto', 'dos', 'unix']
 NEWLINE_CHARS = {
     'auto': None,
