@@ -21,6 +21,7 @@ class CuiApp:
         self.clipboard = ''
         self._quit = False
         self._theme_name = self.DEFAULT_THEME
+        self.last_dir = '.'
 
     def init(self, mainframe):
         self.messagebar = messagebarmode.MessageBarMode()
@@ -34,6 +35,20 @@ class CuiApp:
         self.focus = self.mainframe
         self.macro = macro.Macro()
 
+        self.mainframe.on_console_resized()
+        self.messagebar.set_message(self.SHOW_MENU_MESSAGE)
+
+        self.config.hist_files.load()
+        self.config.hist_dirs.load()
+        self.config.hist_searchstr.load()
+        self.config.hist_replstr.load()
+
+    def on_shutdown(self):
+        self.config.hist_files.save()
+        self.config.hist_dirs.save()
+        self.config.hist_searchstr.save()
+        self.config.hist_replstr.save()
+
     def get_current_theme(self):
         return self._theme_name
 
@@ -144,16 +159,16 @@ class CuiApp:
         return self.mainframe.childframes[:]
 
     def get_activeframe(self):
-        assert self.mainframe.activeframe
         return self.mainframe.activeframe
 
     def run(self):
-        self.mainframe.on_console_resized()
-        self.messagebar.set_message(self.SHOW_MENU_MESSAGE)
 
         nonblocking = True
         while not self._quit:
             try:
+                if not self.focus:
+                    kaa.log.error('Internal error: invalid focus window.')
+                    break
                 inputs = self.focus.do_input(nonblocking)
                 for c in inputs:
                     if isinstance(c, keydef.KeyEvent):
