@@ -18,6 +18,9 @@ class ChildFrame(Window, kaa.context.ContextRoot):
 
     def destroy(self):
         super().destroy()
+        if self.splitter:
+            self.splitter.destroy()
+
         self.destroy_context()
         self.mainframe.childframes.remove(self)
 
@@ -35,6 +38,15 @@ class ChildFrame(Window, kaa.context.ContextRoot):
     def set_active_editor(self, editorwnd):
         self.active_editor = editorwnd
 
+    def get_documents(self):
+        docs = []
+        for splitter, w in self.splitter.walk():
+            if w:
+                doc = w.document
+                if doc not in docs:
+                    docs.append(doc)
+        return docs
+
     def get_title(self):
         if self.splitter:
             return '/'.join(self.splitter.get_title())
@@ -50,17 +62,17 @@ class ChildFrame(Window, kaa.context.ContextRoot):
         super().bring_top()
         if self.splitter:
             self.splitter.bring_top()
-        self.mainframe.activeframe = self
+        self.mainframe.set_activeframe(self)
         self.mainframe.on_console_resized()
 
     def on_setrect(self, l, t, r, b):
         if self.splitter:
             self.splitter.set_rect(l, t, r, b)
 
-
     def on_focus(self):
         super().on_focus()
-        self.mainframe.activeframe = self
+        self.mainframe.set_activeframe(self)
+        
         if self.active_editor:
             self.active_editor.activate()
         elif self.splitter:
@@ -73,7 +85,6 @@ class ChildFrame(Window, kaa.context.ContextRoot):
             self.splitter.draw()
 
 
-
 class MainFrame(Window, kaa.context.ContextRoot):
     """The main frame of application
 
@@ -130,16 +141,47 @@ class MainFrame(Window, kaa.context.ContextRoot):
 
         self.messagebar.set_rect(0, self.height-self.MESSAGEBAR_HEIGHT, self.width, self.height)
 
+    def _get_provisional_frame(self):
+        # if kaa has only one frame and the frame has provisional document,
+        # then dispose the document and reuse the frame.
+
+        frames = kaa.app.get_frames()
+        if len(frames) != 1:
+            return
+
+        wnd = frames[0].splitter.wnd
+        if wnd and wnd.document:
+            doc = wnd.document
+            if doc.provisional and not wnd.document.undo.is_dirty():
+                if len(doc.wnds) == 1:
+                    return frames[0]
+
+    def register_childframe(self, childframe):
+        if childframe in self.childframes:
+            self.childframes.remove(childframe)
+        self.childframes.insert(0, childframe)
+
     def show_doc(self, doc):
-        page = ChildFrame(parent=self)
-        editorwnd = page.show_doc(doc)
+        frame = self._get_provisional_frame()
+        if frame:
+            if frame.splitter.wnd.document is doc:
+                return
+        else:
+            frame = ChildFrame(parent=self)
+
+        editorwnd = frame.show_doc(doc)
         editorwnd.activate()
 
-        self.childframes.append(page)
-        self.activeframe = page
+        self.register_childframe(frame)
+
+        self.set_activeframe(frame)
         self.on_console_resized()
+
         return
 
+    def set_activeframe(self, frame):
+        self.activeframe = frame
+        
     def add_popup(self, popup):
         self.popups.append(popup)
 
