@@ -1,5 +1,5 @@
 import weakref, re
-import gappedbuf, contextlib
+import gappedbuf
 
 
 def is_combine(c):
@@ -20,6 +20,10 @@ class LineNo:
     def lineno(self, pos):
         return self.bisect_left(pos) + 1
 
+    def getpos(self, lineno):
+        if lineno <= len(self.buf):
+            return self.buf.getint(lineno-1)
+
     def bisect_left(self, pos):
         lo = 0
         hi = len(self.buf)
@@ -100,12 +104,10 @@ class Buffer(gappedbuf.GappedBuffer):
 
 class Document:
     all = weakref.WeakSet()  # should not be used!
-
+    provisional = False
     closed = False
     fileinfo = None
 
-    delay_update = False
-
     def __init__(self, buf=None):
         self.wnds = []
         self.all.add(self)
@@ -154,16 +156,6 @@ class Document:
         else:
             self.undo = None
 
-    @contextlib.contextmanager
-    def suspend_update(self):
-        org = self.delay_update
-        self.delay_update = True
-        yield
-        self.delay_update = org
-
-        if not self.delay_update:
-            self.update_screen(0, 0, 0)
-
     def get_title(self):
         title = '<untitled>'
 
@@ -200,8 +192,7 @@ class Document:
             self.styles.delete(pos, pos+dellen)
 
         self.marks.updated(pos, inslen, dellen)
-        if not self.delay_update:
-            self.update_screen(pos, inslen, dellen)
+        self.update_screen(pos, inslen, dellen)
 
     def update_screen(self, pos, inslen, dellen):
         for wnd in self.wnds:
@@ -287,7 +278,13 @@ class Document:
                 break
         return pos
 
+    def get_lineno_pos(self, lineno):
+        # line number starts from one.
 
+        if lineno >= self.buf.lineno.linecount():
+            return self.endpos()
+        else:
+            return self.buf.lineno.getpos(lineno)
 
 
 class Marks(dict):
