@@ -6,12 +6,14 @@ from . import keybind, theme, modebase
 from kaa import highlight
 
 class DefaultMode(modebase.ModeBase):
+    DOCUMENT = True
     MODENAME = 'default'
     SHOW_LINENO = False
     KEY_BINDS = [
         keybind.app_keys,
         keybind.cursor_keys,
         keybind.edit_command_keys,
+            keybind.addtional_edit_command_keys,
         keybind.emacs_keys,
         keybind.search_command_keys,
         keybind.macro_command_keys,
