@@ -39,6 +39,11 @@ cursor_keys = {
     (shift, ctrl, home): ('screen.selection.begin', 'cursor.top-of-file', 'screen.selection.set-end'),
     (shift, ctrl, end): ('screen.selection.begin', 'cursor.end-of-file', 'screen.selection.set-end'),
 
+    (ctrl, 'g'): ('screen.selection.clear', 'cursor.go-to-line'),
+
+    (alt, 'a'): 'screen.selection.all',
+
+    (ctrl, 'c'): 'screen.selection.expand_sel',
 }
 
 
@@ -64,6 +69,9 @@ edit_command_keys = {
     (ctrl, '_'): 'edit.undo',
     (alt, '_'): 'edit.redo',
 
+}
+
+addtional_edit_command_keys = {
     tab: 'edit.indent',
     (shift, tab): 'edit.dedent',
 
