@@ -38,7 +38,7 @@ class ModeBase:
     SCREEN_BUILD_ENTIRE_ROWS = False
     SHOW_LINENO = False
     USE_UNDO = True
-
+    DOCUMENT = False
 
     tab_width = 8
     indent_width = 4
@@ -46,7 +46,7 @@ class ModeBase:
     auto_indent = True
 
     closed = False
-
+    current_lastcommands = None
     theme = None
 
     def __init__(self):
@@ -194,6 +194,12 @@ class ModeBase:
     def on_keypressed(self, wnd, event, s, commands, candidate):
         return s, commands, candidate
 
+    def filter_string(self, wnd, s):
+        return s
+
+    def on_edited(self, wnd):
+        pass
+
     def on_str(self, wnd, s):
         self.edit_commands.put_string(wnd, s)
         if kaa.app.macro.is_recording():
@@ -202,18 +208,23 @@ class ModeBase:
             # run highlighter a bit to display changes immediately.
             self.highlight.update_style(self.document, batch=50)
 
+    def add_lastcommands(self, commandids):
+        if self.current_lastcommands is not None:
+            self.current_lastcommands.extend(commandids)
+
     def on_commands(self, wnd, commandids):
         try:
             if callable(commandids):
                 commandids(wnd)
                 return
 
-            lastcommands = []
+            self.current_lastcommands = []
             for commandid in commandids:
                 is_available, command = self.get_command(commandid)
                 if not command:
                     msg = 'command {!r} is not registered.'.format(commandid)
                     kaa.app.messagebar.set_message(msg)
+                    kaa.log.error(msg)
                     return
 
                 command(wnd)
@@ -221,12 +232,14 @@ class ModeBase:
                     kaa.app.macro.record(command)
 
                 if not getattr(command, 'NORERUN', False):
-                    lastcommands.append(commandid)
-            if lastcommands:
-                kaa.app.lastcommands = lastcommands
+                    self.current_lastcommands.append(commandid)
+
+            if self.current_lastcommands:
+                kaa.app.lastcommands = self.current_lastcommands
 
         finally:
             wnd.editmode.clear_repeat()
+            self.current_lastcommands = None
 
     def on_esc_pressed(self, wnd, event):
         pass
@@ -287,6 +300,14 @@ class ModeBase:
 
         yield from self._split_chars(begin, self.document.endpos())
 
+    def get_word_at(self, pos):
+        tol = self.document.gettol(pos)
+        for f, t, s in self.split_word(tol):
+            if (pos <= f) or (pos < t):
+                return f, t
+
+        return tol, self.document.geteol(tol)
+
     def search_next(self, wnd, pos, searchinfo):
         regex = searchinfo.get_regex()
         pos = min(max(0, pos), self.document.endpos())
