@@ -221,8 +221,9 @@ class Selection:
             self.screen.style_updated()
 
     def get_range(self):
-        if self.start is None or self.end is None:
+        if self.start is None or self.end is None or (self.start == self.end):
             return None
+
         return tuple(sorted((self.start, self.end)))
 
     def set_range(self, f, t):
@@ -290,6 +291,11 @@ class Screen:
 
     def get_total_height(self):
         self.apply_updates()
+
+        if self.build_entire_rows:
+            if self.rows:
+                assert self.rows[0].posfrom == 0
+
         return sum(r.height for r in self.rows)
 
     def setsize(self, width, height):
@@ -523,12 +529,14 @@ class Screen:
         if not refresh and (posidx != -1):
             if not align_always and (self.portfrom <= posidx < self.portto):
                 return False
+
         elif self.build_entire_rows:
             eol, s = self.document.getline(0)
             styles = self.document.get_styles(0, eol)
             self.rows = self._buildrow(0, s, styles)
             self._fillscreen()
             posidx, posrow = self.getrow(pos)
+
         else:
             # build specified row
             tol = self.document.gettol(pos)
@@ -538,6 +546,7 @@ class Screen:
             posidx, posrow = self.getrow(pos)
 
         self.vert_align(posidx, top, middle, bottom)
+
         return True
 
     def vert_align(self, rowidx, top=False, middle=False, bottom=False):
