@@ -1,10 +1,12 @@
 import kaa
+from kaa.keyboard import *
 from kaa.ui.dialog import dialogmode
 from kaa.theme import Theme, Style
 from kaa.filetype.default import modebase, keybind
 from kaa.ui.msgbox import msgboxmode
-from kaa.command import command, Commands
+from kaa.command import command, Commands, norec, norerun
 from kaa.commands import editorcommand
+from kaa.ui.selectlist import filterlist
 
 SearchThemes = {
     'default':
@@ -20,42 +22,82 @@ SearchThemes = {
 }
 
 
-
 class SearchCommands(editorcommand.EditCommands):
 
     @command('edit.backspace')
+    @norec
+    @norerun
     def backspace(self, wnd):
         pos = wnd.cursor.pos
         if pos > wnd.document.marks['searchtext'][0]:
             super().backspace(wnd)
 
     @command('edit.delete')
+    @norec
+    @norerun
     def delete(self, wnd):
         pos = wnd.cursor.pos
         if pos < wnd.document.marks['searchtext'][1]:
             super().delete(wnd)
 
     @command('searchdlg.search.next')
+    @norec
+    @norerun
     def search_next(self, wnd):
         mode = wnd.document.mode
         mode.search_next(wnd)
 
     @command('searchdlg.search.prev')
+    @norec
+    @norerun
     def search_prev(self, wnd):
         mode = wnd.document.mode
         mode.search_prev(wnd)
 
+    
+    def _show_histdlg(self, wnd, title, candidates, callback):
+        doc = filterlist.FilterListInputDlgMode.build(
+                title, callback)
+        dlg = kaa.app.show_dialog(doc)
+    
+        filterlistdoc = filterlist.FilterListMode.build()
+        dlg.add_doc('dlg_filterlist', 0, filterlistdoc)
+        
+        filterlistdoc.mode.set_candidates(candidates)
+    
+        list = dlg.get_label('dlg_filterlist')
+        filterlistdoc.mode.set_query(list, '')
+        dlg.on_console_resized()
+        
+    @command('searchdlg.history')
+    @norec
+    @norerun
+    def search_history(self, wnd):
+        def callback(result):
+            if result:
+                f, t = wnd.document.marks['searchtext']
+                wnd.document.replace(f, t, result)
+            
+        self._show_histdlg(wnd, 'Recent searches', 
+                kaa.app.config.hist_searchstr, callback)
+        
     @command('searchdlg.toggle.ignorecase')
+    @norec
+    @norerun
     def toggle_ignorecase(self, wnd):
         mode = wnd.document.mode
         mode.toggle_option_ignorecase()
 
     @command('searchdlg.toggle.wordsearch')
+    @norec
+    @norerun
     def toggle_wordsearch(self, wnd):
         mode = wnd.document.mode
         mode.toggle_option_wordsearch()
 
     @command('searchdlg.toggle.regex')
+    @norec
+    @norerun
     def toggle_regex(self, wnd):
         mode = wnd.document.mode
         mode.toggle_option_regex()
@@ -63,6 +105,7 @@ class SearchCommands(editorcommand.EditCommands):
 searchdlg_keys = {
     '\r': ('searchdlg.search.next'),
     '\n': ('searchdlg.search.next'),
+    up: ('searchdlg.history'),
 }
 
 class SearchDlgMode(dialogmode.DialogMode):
@@ -82,6 +125,7 @@ class SearchDlgMode(dialogmode.DialogMode):
         self.initialpos = target.cursor.pos
         self.initialrange = target.screen.selection.get_range()
         self.lastsearch = None
+        self.lasthit = None
         self.option = modebase.SearchOption.LAST_SEARCH
 
     def close(self):
@@ -171,7 +215,7 @@ class SearchDlgMode(dialogmode.DialogMode):
         self._build_options(f)
 
         self.update_option_style()
-        kaa.app.messagebar.set_message("Hit alt+N/alt+P to search Next/Prev")
+        kaa.app.messagebar.set_message("Hit alt+N/alt+P to search Next/Prev. Hit up to show history.")
 
     def _set_option_style(self, mark, style,
                           shortcutmark, shortcutstyle):
@@ -220,6 +264,11 @@ class SearchDlgMode(dialogmode.DialogMode):
         f, t = self.document.marks['searchtext']
         return self.document.gettext(f, t)
 
+    def on_edited(self, wnd):
+        self.lastsearch = None
+        self.lasthit = None
+        self._search_next(wnd)
+
     def _show_searchresult(self, hit):
         if hit:
             self.target.cursor.setpos(hit[0])
@@ -227,49 +276,74 @@ class SearchDlgMode(dialogmode.DialogMode):
             self.target.screen.selection.end = hit[1]
             kaa.app.messagebar.set_message('found')
         else:
+            self.target.screen.selection.clear()
             kaa.app.messagebar.set_message('not found')
 
-    def search_next(self, wnd):
+    def _search_next(self, wnd):
         self.option.text = self.get_search_str()
         if self.option.text:
-            if not self.lastsearch:
+            if self.lasthit is self.lastsearch is None:
+                # initial search
                 if self.initialrange:
                     pos = self.initialrange[0]
                 else:
                     pos = self.initialpos
+
+            elif self.lasthit:
+                # continue previous search
+                pos = self.lasthit[0] + 1
+
             else:
-                pos = self.lastsearch[0]+1
+                # resume from top
+                pos = 0
 
             ret = self.target.document.mode.search_next(self.target, pos, self.option)
             self._show_searchresult(ret)
-            if ret:
-                self.lastsearch = ret
-            else:
-                self.lastsearch = (-1, 0)
+
+            self.lastsearch = pos
+            self.lasthit = ret
 
             return ret
 
-    def search_prev(self, wnd):
+    def _save_searchstr(self):
+        kaa.app.config.hist_searchstr.add(self.get_search_str())
+        
+    def search_next(self, wnd):
+        self._save_searchstr()
+        return self._search_next(wnd)
+
+    def _search_prev(self, wnd):
         self.option.text = self.get_search_str()
         if self.option.text:
-            if not self.lastsearch:
+            if self.lasthit is self.lastsearch is None:
+                # initial search
                 if self.initialrange:
                     pos = self.initialrange[1]
                 else:
                     pos = self.initialpos
+
+            elif self.lasthit:
+                # continue previous search
+                pos = self.lasthit[1] - 1
+
             else:
-                pos = self.lastsearch[1]-1
+                # resume from end
+                pos = self.target.document.endpos()
 
             ret = self.target.document.mode.search_prev(self.target, pos, self.option)
             self._show_searchresult(ret)
-            if ret:
-                self.lastsearch = ret
-            else:
-                endpos = self.target.document.endpos()
-                self.lastsearch = (endpos+1, endpos+1)
+
+            self.lastsearch = pos
+            self.lasthit = ret
+
             return ret
 
+    def search_prev(self, wnd):
+        self._save_searchstr()
+        return self._search_prev(wnd)
+
     def on_esc_pressed(self, wnd, event):
+        self._save_searchstr()
         self.target.activate()
         self.target = None
         wnd.get_label('popup').destroy()
@@ -282,8 +356,10 @@ class SearchDlgMode(dialogmode.DialogMode):
                 self.lastsearch = None
 
 
-class ReplaceCommands(Commands):
+class ReplaceCommands(SearchCommands):
     @command('replacedlg.field.next')
+    @norec
+    @norerun
     def field_next(self, wnd):
         searchfrom, searchto = wnd.document.marks['searchtext']
         replacefrom, replaceto = wnd.document.marks['replacetext']
@@ -295,10 +371,31 @@ class ReplaceCommands(Commands):
             wnd.cursor.setpos(searchfrom)
             wnd.screen.selection.set_range(searchfrom, searchto)
 
+    def replace_repl_history(self, wnd):
+        def callback(result):
+            if result:
+                f, t = wnd.document.marks['replacetext']
+                wnd.document.replace(f, t, result)
+            
+        self._show_histdlg(wnd, 'Recent searches', 
+                kaa.app.config.hist_searchstr, callback)
+
+    @command('replacedlg.history')
+    @norec
+    @norerun
+    def replace_history(self, wnd):
+        searchfrom, searchto = wnd.document.marks['searchtext']
+
+        if searchfrom <= wnd.cursor.pos <= searchto:
+            self.search_history(wnd)
+        else:
+            self.replace_repl_history(wnd)
+
 
 replacedlg_keys = {
     '\r': ('replacedlg.field.next'),
     '\n': ('replacedlg.field.next'),
+    up: ('replacedlg.history'),
 }
 
 
@@ -353,7 +450,7 @@ class ReplaceDlgMode(SearchDlgMode):
         self._build_options(f)
 
         self.update_option_style()
-        kaa.app.messagebar.set_message("Hit enter to move field. Hit alt+N/alt+P to replace.")
+        kaa.app.messagebar.set_message("Hit enter to move field. Hit up to show history.`")
 
         return
             
@@ -385,47 +482,66 @@ class ReplaceDlgMode(SearchDlgMode):
         msgdoc = msgboxmode.MsgBoxMode.show_msgbox(
             'Search failed. Resume again?', ['&Yes', '&Cancel'], cb)
 
+    def _save_replstr(self):
+        kaa.app.config.hist_replstr.add(self.get_replace_str())
+        
     def search_next(self, wnd):
         if not self.get_search_str():
             return
 
-        ret = super().search_next(wnd)
-        if ret:
-            self._show_replace_msg(wnd, self.replace_and_next, self.search_next)
+        self._save_searchstr()
+        self._save_replstr()
+        
+        kaa.app.config.hist_searchstr.add(self.get_search_str())
+        
+
+        if self.lastsearch is None:
+            self._search_next(wnd)
+
+        if self.lasthit:
+            self._show_replace_msg(wnd, self.replace_and_next, self.skip_and_next)
         else:
-            self._show_search_again(wnd, self.search_next)
+            self._show_search_again(wnd, self.skip_and_next)
 
     def replace_and_next(self, wnd):
-        if self.lastsearch:
-            f, t = self.lastsearch
-            newstr = self.get_replace_str()
-            self.target.document.mode.edit_commands.replace_string(
-                self.target, f, t, newstr)
+        f, t = self.lasthit
+        newstr = self.get_replace_str()
+        self.target.document.mode.edit_commands.replace_string(
+            self.target, f, t, newstr)
 
-            f, t = self.lastsearch
-            self.lastsearch = (f, f+len(newstr))
-            self.search_next(wnd)
+        self.lasthit = (f, f+len(newstr))
+        self._search_next(wnd)
+        self.search_next(wnd)
+
+    def skip_and_next(self, wnd):
+        self._search_next(wnd)
+        self.search_next(wnd)
 
     def search_prev(self, wnd):
         if not self.get_search_str():
             return
 
-        ret = super().search_prev(wnd)
-        if ret:
-            self._show_replace_msg(wnd, self.replace_and_prev, self.search_prev)
+        self._save_searchstr()
+        self._save_replstr()
+
+        self._search_prev(wnd)
+
+        if self.lasthit:
+            self._show_replace_msg(wnd, self.replace_and_prev, self.skip_and_prev)
         else:
-            self._show_search_again(wnd, self.search_prev)
+            self._show_search_again(wnd, self.skip_and_prev)
 
     def replace_and_prev(self, wnd):
-        if self.lastsearch:
-            f, t = self.lastsearch
-            newstr = self.get_replace_str()
-            self.target.document.mode.edit_commands.replace_string(
-                self.target, f, t, newstr)
+        f, t = self.lasthit
+        newstr = self.get_replace_str()
+        self.target.document.mode.edit_commands.replace_string(
+            self.target, f, t, newstr)
 
-            f, t = self.lastsearch
-            self.lastsearch = (f, f+len(newstr))
-            self.search_prev(wnd)
+        self.lasthit = (f, f+len(newstr))
+        self.search_prev(wnd)
+
+    def skip_and_prev(self, wnd):
+        self.search_prev(wnd)
 
     def replace_all(self, wnd):
         self.option.text = self.get_search_str()
