@@ -8,7 +8,7 @@ from kaa.ui.msgbox import msgboxmode
 
 from kaa.theme import Theme, Style
 
-from kaa.command import Commands, command
+from kaa.command import Commands, command, norec, norerun
 from kaa.keyboard import *
 from kaa.commands import editorcommand
 import kaa.filetype.default.keybind
@@ -64,7 +64,7 @@ class DirFileListMode(selectlist.SelectItemList):
         else:
             self.filterfunc = None
 
-    def show_files(self):
+    def show_files(self, wnd):
         self.cursel = None
         dirs = [selectlist.SelectItem('dirname', name, name)
                     for name in self.dirs]
@@ -100,17 +100,42 @@ class FileOpenDlgCommands(Commands):
         if filename and wnd.document.mode.get_filename() != filename:
             wnd.document.mode.set_filename(wnd, filename)
 
-    @command('fileopendlg.prev')
-    def prev(self, wnd):
+    def _check_needcomplete(self, wnd):
+        cur = wnd.document.mode.get_filename()
+        cur = os.path.expanduser(cur)
+        
+        if os.path.isabs(cur):
+            if os.path.isdir(cur):
+                cur += os.sep
+            self.show_filename(wnd, cur)
+            return True
+        
+        d, f = os.path.split(cur)
+        if d and f:
+            filename = self._build_filename(wnd)
+            self.show_filename(wnd, filename)
+            
+    @command('fileopendlg.next')
+    @norec
+    @norerun
+    def next(self, wnd):
+        if self._check_needcomplete(wnd):
+            return 
+            
         filelist = wnd.get_label('filelist')
-        cursel = filelist.document.mode.sel_prev(filelist)
+        cursel = filelist.document.mode.sel_next(filelist)
         if cursel:
             self._update_filefield(wnd, cursel.value)
 
-    @command('fileopendlg.next')
-    def next(self, wnd):
+    @command('fileopendlg.prev')
+    @norec
+    @norerun
+    def prev(self, wnd):
+        if self._check_needcomplete(wnd):
+            return 
+            
         filelist = wnd.get_label('filelist')
-        cursel = filelist.document.mode.sel_next(filelist)
+        cursel = filelist.document.mode.sel_prev(filelist)
         if cursel:
             self._update_filefield(wnd, cursel.value)
 
@@ -135,13 +160,15 @@ class FileOpenDlgCommands(Commands):
         # set rest of filename to filename filter.
         filelist.document.mode.set_filename(rest)
 
-        filelist.document.mode.show_files()
+        filelist.document.mode.show_files(wnd)
         filelist.get_label('popup').on_console_resized()
 
         # set rest of filename filename field.
         wnd.document.mode.set_filename(wnd, rest)
 
     @command('fileopendlg.openfile')
+    @norec
+    @norerun
     def openfile(self, wnd):
         filename = self._build_filename(wnd)
         if os.path.isfile(filename):
@@ -156,20 +183,22 @@ class FileOpenDlgCommands(Commands):
                 filename += os.path.sep
         self.show_filename(wnd, filename)
 
-class FilenameEditCommand(editorcommand.EditCommands):
+class FilenameEditCommands(editorcommand.EditCommands):
     def on_edited(self, wnd):
+        super().on_edited(wnd)
+
         filename = wnd.document.mode.get_filename()
         if os.sep not in filename:
             filelist = wnd.get_label('filelist')
             filelist.document.mode.set_filename(filename)
-            filelist.document.mode.show_files()
+            filelist.document.mode.show_files(wnd)
             filelist.get_label('popup').on_console_resized()
 
 class OpenFilenameDlgMode(dialogmode.DialogMode):
     MAX_INPUT_HEIGHT = 4
-
+    autoshrink = True
     @classmethod
-    def build(cls, newline, encoding, callback):
+    def build(cls, filename, newline, encoding, callback):
         buf = document.Buffer()
         doc = document.Document(buf)
         mode = cls()
@@ -182,7 +211,7 @@ class OpenFilenameDlgMode(dialogmode.DialogMode):
         f = dialogmode.FormBuilder(doc)
         f.append_text('caption', 'Filename:' )
         f.append_text('default', ' ')
-        f.append_text('default', '', mark_pair='filename')
+        f.append_text('default', filename, mark_pair='filename')
         f.append_text('default', ' ')
 
         f.append_text('option', '[&Encoding:{}]'.format(mode.encoding), mark_pair='enc',
@@ -197,7 +226,6 @@ class OpenFilenameDlgMode(dialogmode.DialogMode):
 
     def close(self):
         super().close()
-        kaa.app.messagebar.set_message("")
 
     def init_themes(self):
         super().init_themes()
@@ -215,7 +243,7 @@ class OpenFilenameDlgMode(dialogmode.DialogMode):
         self.fileopendlg_commands = FileOpenDlgCommands()
         self.register_command(self.fileopendlg_commands)
 
-        self.edit_commands = FilenameEditCommand()
+        self.edit_commands = FilenameEditCommands()
         self.register_command(self.edit_commands)
 
         self.screen_commands = editorcommand.ScreenCommands()
@@ -248,7 +276,9 @@ class OpenFilenameDlgMode(dialogmode.DialogMode):
     def on_esc_pressed(self, wnd, event):
         popup = wnd.get_label('popup')
         popup.destroy()
-
+        kaa.app.messagebar.set_message("")
+        self.callback(None, None, None)
+        
     def get_filename(self):
         f, t = self.document.marks['filename']
         return self.document.gettext(f, t)
@@ -261,6 +291,9 @@ class OpenFilenameDlgMode(dialogmode.DialogMode):
 
     def select_encoding(self, wnd):
         def callback(n):
+            if n is None:
+                return
+
             enc = encodingdef.encodings[n]
             if enc != self.encoding:
                 self.encoding = enc
@@ -279,6 +312,9 @@ class OpenFilenameDlgMode(dialogmode.DialogMode):
 
     def select_newline(self, wnd):
         def callback(n):
+            if n is None:
+                return
+
             nl = consts.NEWLINES[n]
             if nl != self.newline:
                 self.newline = nl
@@ -297,24 +333,31 @@ class OpenFilenameDlgMode(dialogmode.DialogMode):
 
 
 def show_fileopen(filename, callback):
+    if not filename:
+        filename = kaa.app.last_dir
+
     filename = os.path.abspath(
         os.path.expanduser(filename))
 
     if os.path.isdir(filename) and not filename.endswith(os.path.sep):
         filename += os.path.sep
 
-    doc = OpenFilenameDlgMode.build(None, None, callback)
+    doc = OpenFilenameDlgMode.build('', None, None, callback)
     dlg = kaa.app.show_dialog(doc)
 
     filelist = DirFileListMode.build()
     dlg.add_doc('dlg_filelist', 0, filelist)
-    filelist.mode.set_dir('.')
-    filelist.mode.show_files()
+
+    doc.mode.fileopendlg_commands.show_filename(
+        dlg.get_label('editor'), filename)
     dlg.on_console_resized()
+
     return doc
 
 class FileSaveAsDlgCommands(FileOpenDlgCommands):
     @command('fileopendlg.openfile')
+    @norec
+    @norerun
     def openfile(self, wnd):
         filename = self._build_filename(wnd)
 
@@ -353,12 +396,15 @@ class SaveAsFilenameDlgMode(OpenFilenameDlgMode):
 
 
 def show_filesaveas(filename, encoding, newline, callback):
+    if not filename:
+        filename = kaa.app.last_dir
+
     filename = os.path.abspath(
         os.path.expanduser(filename))
     if os.path.isdir(filename) and not filename.endswith(os.path.sep):
         filename += os.path.sep
 
-    doc = SaveAsFilenameDlgMode.build(encoding, newline, callback)
+    doc = SaveAsFilenameDlgMode.build('', encoding, newline, callback)
     dlg = kaa.app.show_dialog(doc)
 
     filelist = DirFileListMode.build()
